/*
 * ConsApp.java
 *
 * Created on 10. Mai 2004, 21:43
 */
package spinfo.io;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
/**
 *
 * @author  sschwieb
 */
public class Application {
      
    /**
     * Dieses Programm veranschaulicht den Geschwindigkeits-
     * Vorteil, der sich aus dem Puffern von Daten ergibt.
     * Das zeichenweise Lesen einer Datei ist relativ langsam,
     * denn für sehr wenig Daten (1 Zeichen) muss hoher Aufwand
     * (Position auf Festplatte suchen o.ä.) betrieben werden.
     * Ein gepufferter Stream sorgt für Abhilfe: Beim ersten
     * Aufruf von "read()" wird nicht nur ein Zeichen gelesen,
     * sondern eine Reihe von Zeichen, die in einem Zwischen-
     * Speicher abgelegt werden. Bei weiteren Aufrufen von
     * read() werden Daten aus dem Zwischenspeicher zurück-
     * gegeben, bis dieser leer ist und erneut eine Reihe
     * von Zeichen aus der eigentlichen Quelle gelesen wird.
     * 
     * Beachten Sie bitte auch den Kommentar zur Objektorientierung
     * an der Methode leseAusReader().
     */
    public static void main(String[] args) {
    	try {
    		System.out.println("Zeitvergleich: Einlesen einer Datei mit verschiedenen Methoden");
    		System.out.println();
			File inputFile = new File("kleist.txt");
			// Liest die Datei auf "naive" Art ein:
			FileReader in1 = new FileReader(inputFile);
			System.out.println("Mit FileReader und StringBuffer:");
			leseAusReader(in1);
			System.out.println();
			in1 = new FileReader(inputFile);
			System.out.println("Mit FileReader und String:");
			readerAufString(in1);
			// Liest die Datei mit Hilfe eines Buffers
			FileReader in2 = new FileReader(inputFile);
			BufferedReader br = new BufferedReader(in2);
			System.out.println();
			System.out.println("Mit BufferedReader und StringBuffer");
			leseAusReader(br);
			in2 = new FileReader(inputFile);
			System.out.println();
			System.out.println("Mit BufferedReader und String");
			readerAufString(in2);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
    }

    /**
     * Diese Methode ist so definiert, dass ein Objekt der Klasse
     * Reader als Parameter erwartet wird. Reader ist die (abstrakte)
     * Superklasse von BufferedReader und FileReader - d.h. ein
     * BufferedReader IST ein Reader (FileReader ebenfalls), und
     * so können Instanzen beider Klassen an diese Methode über-
     * geben werden.
     * Wenn spezifische Methoden einer Klasse nicht benötigt werden
     * (wie z.B. der Konstruktor der Klasse FileReader, der ein File-
     * Objekt benötigt und der von der Klasse BufferedReader nicht an-
     * geboten wird), ist es auch nicht nötig, in einer Methoden-
     * deklaration diese Klasse zu verlangen. Wird stattdessen - wie
     * hier - eine Superklasse gewählt, so wird eine Methode automatisch
     * flexibler, gleichzeitig entfällt die Notwendigkeit, ähnliche
     * Methoden parallel zu warten.
     * Dieses Prinzip nennt sich Substitutionsprinzip und wurde bereits
     * bei der Einführung der Vererbung angesprochen - hier finden Sie
     * nun einmal ein konkretes Beispiel.
     * (siehe auch: http://de.wikipedia.org/wiki/Liskovsches_Substitutionsprinzip)
     */
	private static void leseAusReader(Reader reader) throws IOException, FileNotFoundException{
			// Die Methode currentTimeMillis() der Klasse System
			// ermöglicht es, die Zeit abzurufen - zurückgegeben
			// wird die Anzahl der Millisekunden (1/1000 Sek.),
			// die seit dem 1.Januar 1970 vergangen sind.
    		long start = System.currentTimeMillis();
    		// Das Einlesen funktioniert
    		// wie in den vorherigen Beispielen:
			int character = 0;
			StringBuffer s = new StringBuffer();
			while((character = reader.read()) != -1) {
				s.append((char)character);
			}
			reader.close();
			// Berechnung der benötigten Zeit:
			long end = System.currentTimeMillis();
			long time = end - start;
			System.out.println("Benötigte Zeit: " + time + " Millisekunden.");
	}
	
	private static void readerAufString(Reader reader) throws IOException, FileNotFoundException{
		// Die Methode currentTimeMillis() der Klasse System
		// ermöglicht es, die Zeit abzurufen - zurückgegeben
		// wird die Anzahl der Millisekunden (1/1000 Sek.),
		// die seit dem 1.Januar 1970 vergangen sind.
		long start = System.currentTimeMillis();
		// Das Einlesen funktioniert
		// wie in den vorherigen Beispielen:
		int character = 0;
		String s = "";
		while((character = reader.read()) != -1) {
			s += (char) character;
		}
		reader.close();
		// Berechnung der benötigten Zeit:
		long end = System.currentTimeMillis();
		long time = end - start;
		System.out.println("Benötigte Zeit: " + time + " Millisekunden.");
}
	
	
}
